/* Copyright(C) 2004 Brazil

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _SEN_IO_H
#define _SEN_IO_H

#ifndef _SENNA_H
#include "senna_in.h"
#endif /* _SENNA_H */

#ifdef	__cplusplus
extern "C" {
#endif

enum {
  sen_io_rdonly,
  sen_io_wronly,
  sen_io_rdwr
};

enum {
  sen_io_auto,
  sen_io_manual
};

typedef struct _sen_io sen_io;

typedef struct {
  int32_t segno;
  void *addr;
} sen_io_seginfo;

typedef struct {
  sen_io *io;
  uint32_t mode;
  uint32_t segment;
  uint32_t offset;
  uint32_t size;
  uint32_t nseg;
  void *addr;
} sen_io_win;

sen_io *sen_io_create(const char *path, int header_size, uint32_t segment_size,
		      uint32_t max_segment, int mode, int cache_size);
sen_io *sen_io_open(const char *path, int mode, int cache_size);
sen_rc sen_io_close(sen_io *io);
void *sen_io_header(sen_io *io);

void *sen_io_win_map(sen_io *io, sen_io_win *iw,
		     uint32_t segment, uint32_t offset, uint32_t size, int mode);
sen_rc sen_io_win_unmap(sen_io_win *iw);

void * sen_io_seg_map(sen_io *io, int32_t segno);
sen_rc sen_io_seg_unmap(sen_io *io, int32_t segno);

sen_rc sen_io_seg_bind(sen_io *io, sen_io_seginfo *si);
sen_rc sen_io_seg_unbind(sen_io *io, sen_io_seginfo *si);

#ifdef __cplusplus
}
#endif

#endif /* _SEN_IO_H */
