/* Copyright(C) 2004 Brazil

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <string.h>
#include <ctype.h>
#include "str.h"
#include "lex.h"

/* ngram */

inline static sen_lex *
sen_ngram_open(sen_sym *sym, const char *str, int addp)
{
  sen_lex *lex;
  if (!(lex = SEN_MALLOC(sizeof(sen_lex)))) { return NULL; }
  lex->sym = sym;
  lex->str = str;
  lex->mecab = NULL;
  lex->pos = -1;
  lex->skip = 1;
  lex->tail = 0;
  lex->addp = addp;
  lex->status = sen_lex_doing;
  lex->encoding = sym->encoding;
  if (sym->flags & SEN_INDEX_NORMALIZE) {
    if ((lex->nstr = sen_nstr_open(str, lex->encoding, 1))) {
      lex->str = lex->nstr->norm;
    }
  } else {
    lex->nstr = NULL;
  }
  lex->next = lex->str;
  lex->uni_alpha = (lex->nstr && !(lex->sym->flags & SEN_INDEX_SPLIT_ALPHA));
  lex->uni_digit = (lex->nstr && !(lex->sym->flags & SEN_INDEX_SPLIT_DIGIT));
  lex->uni_symbol = (lex->nstr && !(lex->sym->flags & SEN_INDEX_SPLIT_SYMBOL));
  return lex;
}

inline static sen_id
sen_ngram_next(sen_lex *lex)
{
  char *buf;
  sen_id tid;
  sen_sym *sym = lex->sym;
  uint_least8_t *cp = NULL;
  int32_t len = 0, pos;
  const unsigned char *p, *r, *key = NULL;
  if (lex->status) { return SEN_SYM_NIL; }
  for (p = lex->next, pos = lex->pos + lex->skip; *p; p = r, pos++) {
    if (lex->nstr) { cp = lex->nstr->ctypes + pos; }
    if (lex->uni_alpha && SEN_NSTR_CTYPE(*cp) == sen_str_alpha) {
      for (len = 1, r = p;;len++) {
	r += sen_str_charlen(r, lex->encoding);
	if (SEN_NSTR_ISBLANK(*cp)) { break; }
	if (!*r || SEN_NSTR_CTYPE(*++cp) != sen_str_alpha) { break; }
      }
      if (!(buf = SEN_MALLOC(r - p + 1))) { return SEN_SYM_NIL; }
      memcpy(buf, p, r - p);
      buf[r - p] = '\0';
      tid = lex->addp ? sen_sym_get(sym, buf) : sen_sym_at(sym, buf);
      SEN_FREE(buf);
      if (tid == SEN_SYM_NIL) {
	lex->status = sen_lex_not_found;
	return SEN_SYM_NIL;
      }
      lex->skip = len;
    } else if (lex->uni_digit && SEN_NSTR_CTYPE(*cp) == sen_str_digit) {
      for (len = 1, r = p;;len++) {
	r += sen_str_charlen(r, lex->encoding);
	if (SEN_NSTR_ISBLANK(*cp)) { break; }
	if (!*r || SEN_NSTR_CTYPE(*++cp) != sen_str_digit) { break; }
      }
      if (!(buf = SEN_MALLOC(r - p + 1))) { return SEN_SYM_NIL; }
      memcpy(buf, p, r - p);
      buf[r - p] = '\0';
      tid = lex->addp ? sen_sym_get(sym, buf) : sen_sym_at(sym, buf);
      SEN_FREE(buf);
      if (tid == SEN_SYM_NIL) {
	lex->status = sen_lex_not_found;
	return SEN_SYM_NIL;
      }
      lex->skip = len;
    } else if (lex->uni_symbol && SEN_NSTR_CTYPE(*cp) == sen_str_symbol) {
      for (len = 1, r = p;;len++) {
	r += sen_str_charlen(r, lex->encoding);
	if (SEN_NSTR_ISBLANK(*cp)) { break; }
	if (!*r || SEN_NSTR_CTYPE(*++cp) != sen_str_symbol) { break; }
      }
      if (!(buf = SEN_MALLOC(r - p + 1))) { return SEN_SYM_NIL; }
      memcpy(buf, p, r - p);
      buf[r - p] = '\0';
      tid = lex->addp ? sen_sym_get(sym, buf) : sen_sym_at(sym, buf);
      SEN_FREE(buf);
      if (tid == SEN_SYM_NIL) {
	lex->status = sen_lex_not_found;
	return SEN_SYM_NIL;
      }
      lex->skip = len;
    } else {
      if ((tid = sen_sym_common_prefix_search(sym, p))) {
	key = sen_sym_key(sym, tid);
	len = sen_str_len(key, lex->encoding, NULL);
      }
      r = p + sen_str_charlen(p, lex->encoding);
      // printf("pos=%d len=%d lex->tail\n", pos, len, lex->tail);
      if (tid && (len > 1 || !*r)) {
	if (*r && pos + len - 1 <= lex->tail) { continue; }
	p += strlen(key);
	if (!*p && !lex->addp) { lex->status = sen_lex_done; }
      } else {
	if (lex->addp) {
	  size_t l;
	  char buf2[32], *d = buf2;
	  for (l = sen_str_charlen(p, lex->encoding); l; l--) { *d++ = *p++; }
	  if (*p &&
	      !((lex->uni_alpha && SEN_NSTR_CTYPE(cp[1]) == sen_str_alpha) ||
		(lex->uni_digit && SEN_NSTR_CTYPE(cp[1]) == sen_str_digit) ||
		(lex->uni_symbol && SEN_NSTR_CTYPE(cp[1]) == sen_str_symbol))) {
	    for (l = sen_str_charlen(p, lex->encoding); l; l--) { *d++ = *p++; }
	    len = 2;
	  } else {
	    len = 1;
	  }
	  *d = '\0';
	  tid = sen_sym_get(sym, buf2);
	  // printf("tid=%d,(%s)\n", tid, buf2);
	} else {
	  lex->status = sen_lex_not_found;
	  return SEN_SYM_NIL;
	}
      }
      if ((lex->uni_alpha && SEN_NSTR_CTYPE(cp[len]) == sen_str_alpha) ||
	  (lex->uni_digit && SEN_NSTR_CTYPE(cp[len]) == sen_str_digit) ||
	  (lex->uni_symbol && SEN_NSTR_CTYPE(cp[len]) == sen_str_symbol)) {
	r = p;
	lex->skip = len;
      } else {
	lex->skip = 1;
      }
    }
    lex->pos = pos;
    lex->len = len;
    lex->tail = pos + len - 1;
    lex->next = r;
    // printf("tid=%d pos=%d tail=%d (%s) %s\n", tid, lex->pos, lex->tail, sen_sym_key(sym, tid), r);
    // printf("tid=%d pos=%d tail=%d (%s)\n", tid, lex->pos, lex->tail, sen_sym_key(sym, tid));
    if (!*r) { lex->status = sen_lex_done; }
    return tid;
  }
  lex->status = sen_lex_done;
  return SEN_SYM_NIL;
}

inline static uint_least8_t
sen_ngram_boundary_info(sen_lex *lex)
{
  uint_least8_t *ctypes;
  uint_least8_t pre, head, tail, post, res = 0;
  if (!lex || !lex->nstr || lex->pos < 0) { return 0; }
  ctypes = lex->nstr->ctypes;
  if (lex->pos) {
    head = ctypes[lex->pos];
    pre = ctypes[lex->pos - 1];
    if (SEN_NSTR_ISBLANK(pre) || (pre != SEN_NSTR_CTYPE(head))) {
      res |= 0x02;
    }
  } else {
    res |= 0x02;
  }
  tail = ctypes[lex->tail];
  post = ctypes[lex->tail + 1];
  if (!post || SEN_NSTR_ISBLANK(tail) || (tail != SEN_NSTR_CTYPE(post))) {
    res |= 0x01;
  }
  return res;
}

/* mecab */

// static mecab_t *sole_mecab;

inline static sen_lex *
sen_mecab_open(sen_sym *sym, const char *str, int addp)
{
  // char *arg[] = {"", "-Owakati"};
  char *s, *p;
  sen_lex *lex;
  if (!(lex = SEN_MALLOC(sizeof(sen_lex)))) { return NULL; }
  lex->sym = sym;
  // sen_log("(%s)", str);
  // lex->mecab = sole_mecab;
  if (!(lex->mecab = mecab_new3())) {
  // if (!(lex->mecab = mecab_new(2, arg))) {
    SEN_FREE(lex);
    return NULL;
  }
  lex->pos = -1;
  lex->offset = 0;
  lex->len = 0;
  lex->addp = addp;
  lex->status = sen_lex_doing;
  lex->encoding = sym->encoding;
  if (sym->flags & SEN_INDEX_NORMALIZE) {
    if ((lex->nstr = sen_nstr_open(str, lex->encoding, 0))) {
      str = lex->nstr->norm;
    }
  } else {
    lex->nstr = NULL;
  }
  s = mecab_sparse_tostr(lex->mecab, (char *)str);
  for (p = s + strlen(s) - 1; *p == '\n' || isspace(*p); p--) { *p = '\0'; }
  // sen_log("sparsed='%s'", s);
  lex->str = s;
  lex->next = lex->str;
  return lex;
}

inline static sen_id
sen_mecab_next(sen_lex *lex)
{
  char *buf;
  sen_id tid;
  sen_sym *sym = lex->sym;
  uint_least8_t *cp = NULL;
  int32_t size, len, offset = lex->offset + lex->len;
  const unsigned char *p;
  if (lex->status) { return SEN_SYM_NIL; }
  if (lex->nstr) { cp = lex->nstr->ctypes + offset; }
  for (p = lex->next, len = 0; *p;) {
    p += sen_str_charlen(p, lex->encoding);
    len++;
    /*
    if (cp) {
      if (SEN_NSTR_ISBLANK(*cp++)) { break; }
    } else {
    */
    if (isspace(*p)) { break; }
  }
  if (!len) {
    lex->status = sen_lex_done;
    return SEN_SYM_NIL;
  }
  size = p - lex->next;
  if (!(buf = SEN_MALLOC(size + 1))) { return SEN_SYM_NIL; }
  memcpy(buf, lex->next, size);
  buf[size] = '\0';
  // sen_log("size=%d (%s)", size, buf);
  tid = lex->addp ? sen_sym_get(sym, buf) : sen_sym_at(sym, buf);
  SEN_FREE(buf);
  if (tid == SEN_SYM_NIL) {
    lex->status = sen_lex_not_found;
  } else {
    while (isspace(*p)) { p++; }
    lex->next = p;
    lex->offset = offset;
    lex->len = len;
    if (!*p) { lex->status = sen_lex_done; }
  }
  lex->pos++;
  return tid;
}

inline static uint_least8_t
sen_mecab_boundary_info(sen_lex *lex)
{
  uint_least8_t *ctypes;
  uint_least8_t pre, head, tail, post, res = 0;
  if (!lex || !lex->nstr || lex->pos < 0) { return 0; }
  ctypes = lex->nstr->ctypes;
  if (lex->offset) {
    head = ctypes[lex->offset];
    pre = ctypes[lex->offset - 1];
    if (SEN_NSTR_CTYPE(pre) != SEN_NSTR_CTYPE(head)) {
      res |= 0x02;
    }
  } else {
    res |= 0x02;
  }
  tail = ctypes[lex->offset + lex->len - 1];
  post = ctypes[lex->offset + lex->len];
  if (!post || (SEN_NSTR_CTYPE(tail) != SEN_NSTR_CTYPE(post))) {
    res |= 0x01;
  }
  return res;
}

/* external */

sen_rc
sen_lex_init(void)
{
  char *arg[] = {"", "-Owakati"};
  return mecab_load_dictionary(2, arg) ? sen_success : sen_other_error;
  // sole_mecab = mecab_new(2, arg);
  return sen_success;
}

sen_lex *
sen_lex_open(sen_sym *sym, const char *str, int addp)
{
  if (!(sym->flags & SEN_INDEX_NGRAM)) {
    return sen_mecab_open(sym, str, addp);
  } else {
    return sen_ngram_open(sym, str, addp);
  }
}

sen_rc
sen_lex_next(sen_lex *lex)
{
  if (!(lex->sym->flags & SEN_INDEX_NGRAM)) {
    return sen_mecab_next(lex);
  } else {
    return sen_ngram_next(lex);
  }
}

uint_least8_t
sen_lex_boundary_info(sen_lex *lex)
{
  if (!(lex->sym->flags & SEN_INDEX_NGRAM)) {
    return sen_mecab_boundary_info(lex);
  } else {
    return sen_ngram_boundary_info(lex);
  }
}

sen_rc
sen_lex_close(sen_lex *lex)
{
  if (!lex) { return sen_invalid_argument; }
  if (lex->nstr) { sen_nstr_close(lex->nstr); }
  if (lex->mecab) { mecab_destroy(lex->mecab); }
  SEN_FREE(lex);
  return sen_success;
}
