/* Copyright(C) 2004 Brazil

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _SEN_STR_H
#define _SEN_STR_H

#ifndef _SENNA_H
#include "senna_in.h"
#endif /* _SENNA_H */

#ifdef	__cplusplus
extern "C" {
#endif

enum {
  sen_str_null = 0,
  sen_str_alpha,
  sen_str_digit,
  sen_str_symbol,
  sen_str_hiragana,
  sen_str_katakana,
  sen_str_kanji,
  sen_str_others
};

extern const char * sen_enc_string[];

typedef struct {
  uint32_t ctype;
  char *start;
  uint32_t length;
} sen_substr;

typedef struct _sen_str_splitter sen_str_splitter;

typedef struct {
  const char *orig;
  char *norm;
  uint_least8_t *ctypes;
  int length;
  sen_encoding encoding;
} sen_nstr;

sen_str_splitter *sen_str_splitter_open(char *str);
char *sen_str_splitter_next(sen_str_splitter *s, int *ctype);
sen_rc sen_str_splitter_close(sen_str_splitter *s);

int sen_str_split(char *str, sen_substr *res, int max, char **rest);

int sen_str_get_prefix_order(const char *str);


sen_nstr * sen_nstr_open(const char *str, sen_encoding encoding, int removeblankp);
sen_rc sen_nstr_close(sen_nstr *nstr);

size_t sen_str_charlen(const char *str, sen_encoding encoding);
size_t sen_str_len(const char *str, sen_encoding encoding, const char **last);
sen_rc sen_str_fin(void);

#define SEN_NSTR_BLANK 0x80
#define SEN_NSTR_ISBLANK(c) (c & 0x80)
#define SEN_NSTR_CTYPE(c) (c & 0x7f)

#ifdef __cplusplus
}
#endif

#endif /* _SEN_STR_H */
