/* Copyright(C) 2004 Brazil

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "inv.h"
#include "sym.h"
#include "str.h"

/* copy from inv.c */

#define SEN_INV_IDSTR "SENNA:INV:00.00"
#define SEN_INV_HEADER_SIZE  0x10000
#define SEN_INV_SEGMENT_SIZE 0x40000
#define SEN_INV_CHUNK_SIZE   0x40000
#define FILESIZE_MAX 0x80000000
#define MAX_SEGMENT (FILESIZE_MAX / SEN_INV_SEGMENT_SIZE)
#define MAX_ID (FILESIZE_MAX / sizeof(uintptr_t) - 1)
#define W_OF_SEGMENT 18
#define W_OF_ARRAY (W_OF_SEGMENT - 2)
#define ARRAY_MASK_IN_A_SEGMENT (SEN_INV_SEGMENT_SIZE / sizeof(uintptr_t) - 1)
#define BUFFER_MASK_IN_A_SEGMENT (SEN_INV_SEGMENT_SIZE - 1)
#define NCHUNK_IN_A_FILE (FILESIZE_MAX / SEN_INV_CHUNK_SIZE)
#define CHUNK_NOT_ASSIGNED MAX_SEGMENT
#define SEGMENT_ARRAY 0x8000
#define SEGMENT_BUFFER 0x4000
#define SEGMENT_MASK (MAX_SEGMENT - 1)

struct sen_inv_header {
  char idstr[16];
  uint16_t segments[MAX_SEGMENT];
  uint8_t chunks[MAX_SEGMENT];
  uint32_t initial_n_segments;
};

struct _sen_inv {
  sen_io *seg;
  sen_io *chunk;
  struct sen_inv_header *header;
  sen_io_seginfo ainfo[MAX_SEGMENT];
  sen_io_seginfo binfo[MAX_SEGMENT];
};

typedef struct {
  uint32_t tid;
  uint32_t size_in_chunk;
  uint32_t pos_in_chunk;
  uint32_t pos_in_buffer;
} buffer_term;

typedef struct {
  uint32_t var0;
  uint32_t var1;
} buffer_rec;

typedef struct {
  uint32_t chunk;
  uint32_t chunk_size;
  uint32_t buffer_free;
  uint32_t nterms;
} buffer_header;

typedef struct {
  buffer_header header;
  buffer_term terms[(SEN_INV_SEGMENT_SIZE - sizeof(buffer_header))/sizeof(buffer_term)];
} buffer;

void
sen_inv_describe(sen_inv *inv)
{
  int i, j, n = 0;
  puts("chunks");
  for (i = 0; i < 128; i++) {
    for (j = 0; j < 64; j++) {
      printf("%d", inv->header->chunks[i * 64 + j]);
      n += inv->header->chunks[i * 64 + j];
    }
    puts(":");
  }
  printf("n=%d\n", n);
  puts("segments");
  {
    int a = 0, b = 0;
  for (i = 0; i < 128; i++) {
    for (j = 0; j < 64; j++) {
      if (inv->header->segments[i * 64 + j] & SEGMENT_ARRAY) {
	a++;
      }
      if (inv->header->segments[i * 64 + j] & SEGMENT_BUFFER) {
	b++;
      }
      printf("%x ", inv->header->segments[i * 64 + j]);
    }
    puts(":");
  }
  printf("a=%d, b=%d\n", a, b);
  }
}

void
checkdb(sen_index *i)
{
  int j;
  buffer *b;
  sen_inv *inv = i->inv;
  sen_io_seginfo *si;
  unsigned int size = 0;
  for (j = 0; j < 1024; j++) {
    si = &inv->binfo[j];
    if (sen_io_seg_bind(inv->seg, si)) { continue; }
    b = si->addr;
    if (b->header.chunk != CHUNK_NOT_ASSIGNED) {
      printf("j=%d c=%d s=%d f=%d n=%d\n",
	     j,
	     b->header.chunk,
	     b->header.chunk_size,
	     b->header.buffer_free,
	     b->header.nterms);
    }
    size += b->header.chunk_size;
  }
  printf("total size = %u\n", size);
}

int
main(int argc, char **argv)
{
  sen_index *i;
  int key_size = sizeof(int);
  if (argc < 3) {
    puts("Usage: itest get|put|whole indexfile");
    return 1;
  }
  if (!(i = sen_index_open(argv[2]))) {
    i = sen_index_create(argv[2], key_size, SEN_INDEX_NORMALIZE, 0, sen_enc_default);
  }
  if (!i) { fputs("index open failed\n", stderr); }
  sen_inv_describe(i->inv);
  checkdb(i);
  return 0;
}
