/* Copyright(C) 2004 Brazil

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#define _XOPEN_SOURCE 500

#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/stat.h>

char *
do_mmap(int fd, int size)
{
  char *p = mmap(NULL, size, PROT_READ, MAP_PRIVATE, fd, 0);
  if (p == MAP_FAILED) {
    fputs(strerror(errno), stderr);
    return NULL;
  }
  return p;
}

char *
do_share_mmap(int fd, int size)
{
  char *p = mmap(NULL, size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
  if (p == MAP_FAILED) {
    fputs(strerror(errno), stderr);
    return NULL;
  }
  return p;
}

char *
do_pread(int fd, int size)
{
  char *p = malloc(size);
  int r = pread(fd, p, size, 0);
  if (r != size) {
    fputs(strerror(errno), stderr);
  }
  return p;
}

char *
do_read(int fd, int size)
{
  char *p = malloc(size);
  int r = read(fd, p, size);
  if (r != size) {
    fputs(strerror(errno), stderr);
  }
  return p;
}

int
main(int argc, char **argv)
{
  int fd, i;
  char *p, *q;
  struct stat s;
  if (argc < 3) {
    puts("Usage: mtest mmap|read|pread datafile");
    return 1;
  }
  if ((fd = open(argv[2], O_RDWR)) == -1) { return 1; }
  if (fstat(fd, &s) == -1) { return -1; }
  switch (*argv[1]) {
  case 'm' :
    p = do_mmap(fd, s.st_size);
    break;
  case 's' :
    p = do_share_mmap(fd, s.st_size);
    break;
  case 'p' :
    p = do_pread(fd, s.st_size);
    break;
  case 'r' :
    p = do_read(fd, s.st_size);
    break;
  default :
    return 1;
  }
  q = p + s.st_size;
  for (i = 0; p < q; p++) {
    i += *p;
  }
  printf("%d\n", i);
  // free(buffer);
  // close(fd);
  return 0;
}
